#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os,atexit,signal,subprocess,time,shlex
from sys import *
import pygtk
pygtk.require("2.0")
import gtk,gobject
from gtk import gdk
try:
    from xfce4 import netk as wnck
except ImportError:
    import wnck
try:
    import hotkey
except ImportError:
    pass
    #print >>sys.stderr,"Can't import hotkey modules.. Maybe, python-xlib is missing?"

#try:
#    from qconserv import Qconserv
#except ImportError:
#    Qconserv=lambda arg: None
#    debug("Can't import dbus service module..")




from conf import Conf
conf=Conf('~/.qconrc')





def handle_signal(*args):
    term_instance.toggle_window()

def reload_conf(*args):
    term_instance.window.setup(True)

def terminate(*args):
    term_instance.kill()

if '-v' in argv:
    def debug(*msg):
        msg=" ".join(map(str,msg))
        print >>stderr,msg
else:
    def debug(*msg):
        pass



class PidFile(object):
    filename = property(lambda self:os.path.expanduser('~/.qcon.pid'))
    togglesh = property(lambda self:os.path.expanduser('~/.qcon-toggle'))
    def __init__(self):
        self.pid=os.getpid()
        try: oldpid=int(open(self.filename).readline().strip())
        except: oldpid=None
        if oldpid:
            try:
                os.kill(oldpid,0)
                print >>stderr,'Already running'
                exit(1)
            except OSError:
                pass
        open(self.filename,'w').write(str(self.pid))
        open(self.togglesh,'w').write("#!/bin/sh\nexec kill -USR1 %d\n" % self.pid)
        os.chmod(self.togglesh,0755)
        atexit.register(self.unlink)
    def unlink(self):
        debug('Removing pid file')
        os.unlink(self.filename)
        open(self.togglesh,'w').write("#!/bin/sh\nqcon\n")
        os.chmod(self.togglesh,0755)

class Term(object):
    def __init__(self,cmd):
        self._cmd=cmd
        self.screen=wnck.screen_get_default()
        self.run()
        if not globals().has_key('term_instance'):
            global term_instance
            term_instance=self
            signal.signal(signal.SIGUSR1,handle_signal)
            atexit.register(Term.kill,self)
        if conf.KEY:
            try:
                import hotkey
                hotkey=hotkey.Hotkey(conf.KEY,handle_signal)
            except ImportError:
                debug("Can't import hotkey modules.. Maybe, python-xlib is missing?")
    def run(self):
        self.proc=subprocess.Popen(tuple(shlex.split(self._cmd)))
        self.window=None
        self._watch_id=gobject.child_watch_add(self.proc.pid,self.on_child_exit)
        gobject.idle_add(self.find_window)
        self._find_timeout_id=gobject.timeout_add(conf.FIND_WINDOW_TIMEOUT*1000,self.find_timeout)
    def on_child_exit(self,cpid,errlev):
        debug('Terminal process with pid %d died with exit status %d'%(cpid,errlev))
        self.proc.returncode=errlev
        if conf.RESPAWN:
            self.run()
        else:
            exit(errlev)
    def find_window(self):
        for w in self.screen.get_windows():
            if w.get_pid() == self.proc.pid:
                self.window=TermWindow(w,self)
                gobject.source_remove(self._find_timeout_id)
                return False
        return True
    def find_timeout(self):
        if not self.window:
            print >>stderr,"No terminal window for %d seconds, terminating" % conf.FIND_WINDOW_TIMEOUT
            exit(3)
    def toggle_window(self):
        self.window and self.window.toggle()
    def kill(self):
        gobject.source_remove(self._watch_id)
        if self.proc.poll() is None:
            debug('Killing terminal process')
            try:
                os.kill(self.proc.pid,signal.SIGINT)
            except OSError:
                pass

    
class TermWindow(object):
    def __init__(self,window,termproc):
        self.term=termproc
        self.window=window
        self.wrapped=gdk.window_foreign_new(window.get_xid())
        #self.toolbar=ToolbarWindow(self.wrapped)
        self.focus=self.focus_wnck if wnck.__name__=='wnck' else self.focus_netk
        self.setup()
        self._active=False
        self.term.screen.connect('active_window_changed',self.hide_inactive)
        self.hide_inactive()
        if conf.START_HIDDEN:
            gobject.idle_add(self.hide)
    def setup(self,reload_conf=False):
        global conf
        if reload_conf:
            conf=Conf('~/.qconrc') #TODO if conf.COMMAND!=oldcmd: Dialog
        #self.wrapped.set_skip_pager_hint(True) ## causes segfault in ubuntu 8.10
        #self.wrapped.set_keep_above(True)
        self.wrapped.set_decorations(conf.BORDER)
        self.wrapped.set_functions(0 | gdk.FUNC_MINIMIZE)
        self.wrapped.move(*(conf._geom(self.term.screen)[0]))
        self.wrapped.resize(*(conf._geom(self.term.screen)[1]))
        ###gobject.idle_add(self.toolbar.stick)
        try:
            icon=conf._getpath('qcon.png','pixmaps')
            #self.wrapped.set_icon_list([gtk.gdk.pixbuf_new_from_file(icon),])
        except:
            print >>stderr,"Can't set icon"
        if reload_conf:
            self.activate()
    def activate(self):
        #self.wrapped.set_skip_taskbar_hint(False)
        self.wrapped.deiconify()
        #self.wrapped.set_keep_above(True)
        self.wrapped.raise_()
        self.focus()
        self._active=True
        #self.toolbar.show()
    def hide(self):
        self.wrapped.iconify()
        #self.wrapped.set_skip_taskbar_hint(True)
        self._active=False
        #self.toolbar.hide()
    def toggle(self):
        if not self.wrapped.get_state() & gdk.WINDOW_STATE_ICONIFIED:#self.window.is_active():
            debug('toggle:hide');self.hide()
        else:
            debug('toggle:activate');self.activate()
    def hide_inactive(self,scrobj=None,*args):
        active_pid=self.term.screen.get_active_window()
        if not self._active and active_pid==self.window:
            self.activate()
        elif self._active:
            active_pid=active_pid and active_pid.get_pid() or 0
            if active_pid != self.term.proc.pid:
                debug('hide_inactive')
                self.hide()
        return False
    def focus_wnck(self):
        self.wrapped.focus()
        self.window.activate(gtk.get_current_event_time())
    def focus_netk(self):
        self.wrapped.focus()
        self.window.activate()






def main():
    global conf
    if not conf._exists:
        import settings
        settings.main()
        conf=Conf('~/.qconrc')

    pidfile=PidFile()
    signal.signal(signal.SIGHUP,reload_conf)
    term=Term(conf.COMMAND)

    loop=gobject.MainLoop()
    loop.run()

def show_usage():
    print open(conf._getpath('USAGE','doc')).read()

if __name__=='__main__':
    main()

